@@ -1,4 +1,11 @@
 """A data wrangling tool which requires no knowledge of Pandas or SQL.
+
+*** Type definitions ***
+
+rowval = str/int/float
+    (slash(/) means `or`)
+row = dict[str, rowval]
+    lowercase, row represents a row in a table, str is a column name (field)
 """
 
 import os
@@ -16,8 +23,9 @@ from pathlib import Path, PurePath
 class Conn:
     """Connection to a SQL database file.
 
-    Example:
-        conn = Conn('sample.db')
+    Examples
+
+    >>> conn = Conn('sample.db')
     """
 
     def __init__(self, dbfile):
@@ -29,14 +37,34 @@ class Conn:
 
         self._dbconn = None
 
-    def __getitem__(self, tname):
-        return Rows((self, tname))
+    def __getitem__(self, table_name):
+        """Returns a Rows object to fetch a table from the database
+
+        Parameters
+        ----------
+        table_name: str
+
+        Returns
+        -------
+        Rows
+        """
+        return Rows((self, table_name))
+
+    def __setitem__(self, table_name, val):
+        """Insert the table content to the database
+
+        Parameters
+        ----------
+        table_name: str
+
+        val: Rows/list[row]/Iterable[row]
+            row = dict[str, int/float/str]
+        """
 
-    def __setitem__(self, tname, val):
         with _connect(self._dbfile) as dbconn:
             self._dbconn = dbconn
-            _delete(dbconn, tname)
-            _insert(dbconn, tname, _rows2iter(val))
+            _delete(dbconn, table_name)
+            _insert(dbconn, table_name, _rows2iter(val))
 
 
 # TODO: Not sure whether to check the syntax for each method.
@@ -46,10 +74,15 @@ class Rows:
     builds instructions as methods growing in chained
 
     Rows object can be instantiated in two ways.
+
         1. to pass a table name to Conn.__getitem__, for example,
-            conn['table_name']
-        2. Directly pass a list of dicts or dicts yielding iterator to Rows
-            Rows([d1, d2, ...])
+
+            >>> conn['table_name']
+
+        2. Directly pass a list of dicts(list[dict]) or dicts yielding iterator
+        (Iterable[dict])
+
+            >>> Rows([d1, d2, ...])
     """
 
     def __init__(self, src):
@@ -80,6 +113,23 @@ class Rows:
         return "\n".join(str(r) for r in self._iter())
 
     def __getitem__(self, key):
+        """Two different ways of getting items
+
+        Parameters
+        ----------
+        key: str/slice
+
+        Returns
+        -------
+        list[rowval]/Rows
+
+            if isinstance(key, str) returns list[rowval]
+
+            if isinstance(key, slice) returns Rows
+
+            slicing is similar to list slicing except that negative values
+            cannot be used
+        """
         if isinstance(key, str):
             return [r[key] for r in self._iter()]
         if isinstance(key, slice):
@@ -89,11 +139,13 @@ class Rows:
     def zip(self, other):
         """Attach two Rows side by side until either one is depleted
 
-        Args:
-            other (Rows or list of dicts or dicts yielding iterator)
+        Parameters
+        ----------
+        other: Rows/list[row]/Iterable[row]
 
-        Returns:
-            Rows
+        Returns
+        -------
+        Rows
         """
         newself = copy(self)
         # pylint: disable=protected-access
@@ -124,11 +176,13 @@ class Rows:
 
         Empty columns (with values of '') generated
 
-        Args:
-            other (Rows or list of dicts or dicts yielding iterator)
+        Parameters
+        ----------
+        other: Rows/list[row]/Iterable[row]
 
-        Returns:
-            Rows
+        Returns
+        -------
+        Rows
         """
 
         newself = copy(self)
@@ -178,11 +232,13 @@ class Rows:
     def chain(self, other):
         """Concatenate Rows to the other
 
-        Args:
-            other (Rows or list of dicts or dicts yielding iterator)
+        Parameters
+        ----------
+        other: Rows/list[row]/Iterable[row]
 
-        Returns:
-            Rows
+        Returns
+        -------
+        Rows
         """
         newself = copy(self)
         # pylint: disable=protected-access
@@ -206,11 +262,16 @@ class Rows:
     def order(self, *fields_maybe_with_desc):
         """Sort according to columns
 
-        Args:
-            *cols (str): column names
+        Parameters
+        ----------
+        *fields_maybe_with_desc: list[str]
 
-        Returns:
-            Rows object
+            str is a field name but it can be with `desc` keyword, for example,
+            'col desc'
+
+        Returns
+        -------
+        Rows
         """
         newself = copy(self)
 
@@ -267,7 +328,7 @@ class Rows:
         # pylint: disable=protected-access
         newself._history = newself._history + [{
             'cmd': 'order',
-            'cols': fields_maybe_with_desc,
+            'fields': fields_maybe_with_desc,
             'genfn': gen_from_sql if newself._history[0]['cmd'] == 'fetch'
             else gen_simp
         }]
@@ -313,11 +374,13 @@ class Rows:
     def group(self, *fields):
         """Group consecutive rows with the same values for given fields
 
-        Args:
-            *fields (list of str)
+        Parameters
+        ----------
+        *fields: list[str]
 
-        Returns:
-            Rows
+        Returns
+        -------
+        Rows
         """
         newself = copy(self)
         # pylint: disable=protected-access
@@ -330,7 +393,7 @@ class Rows:
         # pylint: disable=protected-access
         newself._history = newself._history + [{
             'cmd': 'group',
-            'cols': fields,
+            'fields': fields,
             'genfn': gen
         }]
 
@@ -357,16 +420,16 @@ class Rows:
     def map(self, func):
         """Applies func to each row or Rows (when grouped)
 
-        Args:
-            func: args: row(dict) when not grouped or
-                        Rows when grouped
-                             (preceded by 'by', 'windowed', 'group')
-                  returns: row (dict) or
-                           Rows or
-                           None
+        Parameters
+        ----------
+        func: row/Rows -> row/Rows/None
 
-        Returns:
-            Rows
+            takes row (dict(str, int/float/str) when not grouped
+            takes Rows when grouped by methods such as by, group, windowed
+
+        Returns
+        -------
+        Rows
         """
         if self._history[-1]['cmd'] == 'group':
             def _fn2gen(func):
@@ -377,6 +440,8 @@ class Rows:
                     elif isinstance(val, Rows):
                         # pylint: disable=protected-access
                         yield from val._iter()
+                    # if it's neither dict nor Rows simply ignored.
+                    # no Nonetype check
                 return gen
             return self._apply(_fn2gen(func))
 
@@ -411,11 +476,16 @@ class Rows:
 
         order columns first and then group
 
-        Args:
-            *fields_maybe_with_desc: "col1" or "col1 desc"
+        Parameters
+        ----------
+        *fields_maybe_with_desc: list[str]
 
-        Returns:
-            Rows
+            str is a field name but it can be with `desc` keyword, for example,
+            'col desc'
+
+        Returns
+        -------
+        Rows
         """
         # need to cut out 'desc', 'asc'
         grouping_fields = [field.split()[0]
@@ -425,11 +495,13 @@ class Rows:
     def filter(self, pred):
         """Filter rows that the predicate function returns True
 
-        Args:
-            pred: function that takes a row(dict) as arg and returns True/False
+        Parameters
+        ----------
+        pred: row -> bool
 
-        Returns:
-            Rows
+        Returns
+        -------
+        Rows
         """
         def gen(row):
             if pred(row):
@@ -444,16 +516,13 @@ class Rows:
         kwargs are applied one by one. The next key value pair uses the
         previously updated row
 
-        Args:
-            **kwargs: key = new field name
-                      value = 1. function
-                                    args: row(dict)
-                                    returns: str or number
-                              2. str or number
-
-        Returns:
-            Rows
+        Parameters
+        ----------
+        **kwargs: dict[str, row -> rowval]
 
+        Returns
+        -------
+        Rows
         """
 
         def updatefn1(row):
@@ -477,12 +546,17 @@ class Rows:
     def rename(self, **kwargs):
         """Rename fields with new ones
 
-        Args:
-            **kwargs: key = new field name
-                      value = old field name (str)
+        Parameters
+        ----------
+        **kwargs: dict[str, str]
 
-        Returns:
-            Rows
+            key str(new field name), value str(old field name)
+
+            >>> rows.rename(new_field_name='old_field_name')
+
+        Returns
+        -------
+        Rows
         """
         kwargs_rev = {oldkey: newkey for newkey, oldkey in kwargs.items()}
 
@@ -498,19 +572,18 @@ class Rows:
         fold must be preceded by grouping methods (group, by, windowed) and
         each group is shrunken to 1
 
-        Args:
-            **kwargs: key = new field name
-                      value = 1. function
-                                    args: Rows
-                                    returns: str or number
-                              2. str or number
-        Returns:
-            Rows
+        Parameters
+        ----------
+        **kwargs: dict[str, Rows -> rowval]
+
+        Returns
+        -------
+        Rows
         """
         # fold must be preceded by `by`
         grouping_fields = []
         if self._history[-1]['cmd'] == 'group':
-            grouping_fields = self._history[-1]['cols']
+            grouping_fields = self._history[-1]['fields']
 
         # TODO: not sure whether to check if func returns a proper value
         # (a str or a number)
@@ -538,11 +611,12 @@ class Rows:
     def select(self, *fields):
         """select fields discarding others
 
-        Args:
-            *fields(list of str): field names to select
+        Parameters
+        ----------
+        *fields: list[str]
 
         Returns:
-            Rows
+        Rows
         """
         def selectfn(row):
             yield {field: row[field] for field in fields}
@@ -552,11 +626,12 @@ class Rows:
     def deselect(self, *fields):
         """Deselect fields
 
-        Args:
-            *fields(list of str): field names to deselect
+        Parameters
+        ----------
+        *fields: list[str]
 
         Returns:
-            Rows
+        Rows
         """
         fields_set = set(fields)
 
@@ -569,14 +644,20 @@ class Rows:
     def join(self, other, join_type="left"):
         """Joins two Rows in SQL style
 
-        Args:
-            Other: Rows or a list of rows(dicts) or row(dict) yielding iterator
-            join_type: one of 'inner', 'left', 'right', 'full',
-                each representing SQL style join types. 'left' is the default
+        Parameters
+        ----------
+        other: Rows/list[row]/Iterable[row]
 
-        Returns:
-            Rows
+        join_type: str
+
+            'inner', 'left', 'right', 'full'
+            default: 'left'
+
+        Returns
+        -------
+        Rows
         """
+
         if join_type.lower() == "inner":
             return self._inner_join(other)
         if join_type.lower() == "left":
@@ -701,11 +782,13 @@ class Rows:
         Order and group with fields drop all the others except for the first
         in each group (remove duplicates)
 
-        Args:
-            *fields (list of str):
+        Parameters
+        ----------
+        *fields: list[str]
 
-        Returns:
-            Rows
+        Returns
+        -------
+        Rows
         """
         def distinctfn(_k, rows):
             # impossible to raise stop iteration here
@@ -719,12 +802,19 @@ class Rows:
         """Returns a Rows of a sliding window(rows) with chunk_size skipping
         every step size rows
 
-        Args:
-            chunk_size (int)
-            step (int):  1 is the default
+        Parameters
+        ----------
+        chunk_size: int
 
-        Returns:
-            Rows
+            positive integer
+
+        step: int
+
+            positive integer, default 1
+
+        Returns
+        -------
+        Rows
         """
         newself = copy(self)
         # pylint: disable=protected-access
@@ -763,11 +853,14 @@ class Rows:
         """Returns a Rows that would generate rows(dicts) as long as the pred
         is true
 
-        Args:
-            pred: predicate func that takes a row(dict) as an argument
+        Parameters
+        ----------
+        pred: row -> bool
+
+        Returns
+        -------
+        Rows
 
-        Returns:
-            Rows
         """
         newself = copy(self)
         # pylint: disable=protected-access
@@ -788,13 +881,14 @@ class Rows:
         """Returns a Rows that would drop rows as long as the pred
         is true
 
-        Args:
-            pred: predicate func that takes a row(dict) as an argument
+        Parameters
+        ----------
+        pred: row -> bool
 
-        Returns:
-            Rows
+        Returns
+        -------
+        Rows
         """
-
         newself = copy(self)
         # pylint: disable=protected-access
         pgen = newself._history[-1]['genfn']
@@ -811,7 +905,13 @@ class Rows:
         return newself
 
     def size(self):
-        "Returns the number(size) of rows"
+        """Returns the number(size) of rows
+
+        Returns
+        -------
+        int
+        """
+
         # There are rooms for this to be more efficient because some of the
         # methods do not affect the size of the Rows but
         # don't think it's worth the trouble.
@@ -830,7 +930,12 @@ class Rows:
 
     # list is safe, because it completes the iteration.
     def list(self):
-        "Returns a list of rows(dicts)"
+        """Returns a list of rows(dicts)
+
+        Returns
+        -------
+        list[row]
+        """
         # pylint: disable=protected-access
         return list(self._iter())
 
